@@ -1,4 +1,4 @@
-# Copyright 2022-2024 MetaOPT Team. All Rights Reserved.
+# Copyright 2022-2025 MetaOPT Team. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,37 +14,21 @@
 # ==============================================================================
 """Registry for custom pytree node types."""
 
+# pylint: disable=too-many-lines
+
 from __future__ import annotations
 
 import contextlib
 import dataclasses
-import functools
 import inspect
 import sys
-import warnings
 from collections import OrderedDict, defaultdict, deque, namedtuple
 from operator import itemgetter, methodcaller
 from threading import Lock
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    ClassVar,
-    Collection,
-    Generator,
-    Generic,
-    Iterable,
-    NamedTuple,
-    Sequence,
-    Type,
-    TypeVar,
-    overload,
-)
-from typing_extensions import TypeAlias  # Python 3.10+
-from typing_extensions import deprecated  # Python 3.13+
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, Generic, NamedTuple, TypeVar, overload
 
-from optree import _C
-from optree.accessor import (
+import optree._C as _C
+from optree.accessors import (
     AutoEntry,
     MappingEntry,
     NamedTupleEntry,
@@ -52,21 +36,18 @@ from optree.accessor import (
     SequenceEntry,
     StructSequenceEntry,
 )
-from optree.typing import (
-    CustomTreeNode,
-    PyTreeKind,
-    T,
-    is_namedtuple_class,
-    is_structseq_class,
-    structseq,
-)
+from optree.typing import PyTreeKind, StructSequence, T, is_namedtuple_class, is_structseq_class
 from optree.utils import safe_zip, total_order_sorted, unzip2
 
 
 if TYPE_CHECKING:
     import builtins
+    from collections.abc import Collection, Generator, Iterable
 
-    from optree.typing import KT, VT, FlattenFunc, UnflattenFunc
+    from optree.typing import KT, VT, CustomTreeNode, FlattenFunc, UnflattenFunc
+
+    # pylint: disable-next=invalid-name
+    CustomTreeNodeType = TypeVar('CustomTreeNodeType', bound=type[CustomTreeNode])
 
 
 __all__ = [
@@ -88,7 +69,7 @@ class PyTreeNodeRegistryEntry(Generic[T]):
     flatten_func: FlattenFunc[T]
     unflatten_func: UnflattenFunc[T]
 
-    if sys.version_info >= (3, 10):
+    if sys.version_info >= (3, 10):  # pragma: >=3.10 cover
         _: dataclasses.KW_ONLY  # Python 3.10+
 
     path_entry_type: builtins.type[PyTreeEntry] = AutoEntry
@@ -103,7 +84,7 @@ del SLOTS
 class GlobalNamespace:  # pragma: no cover
     __slots__: ClassVar[tuple[()]] = ()
 
-    def __repr__(self) -> str:
+    def __repr__(self, /) -> str:
         return '<GLOBAL NAMESPACE>'
 
 
@@ -121,33 +102,134 @@ if TYPE_CHECKING:
     _GetT = TypeVar('_GetT')
 
     class _CallableWithGet(Generic[_P, _T, _GetP, _GetT]):
-        def __call__(self, *args: _P.args, **kwargs: _P.kwargs) -> _T:
+        def __call__(self, /, *args: _P.args, **kwargs: _P.kwargs) -> _T:
             raise NotImplementedError
 
         # pylint: disable-next=missing-function-docstring
-        def get(self, *args: _GetP.args, **kwargs: _GetP.kwargs) -> _GetT:
+        def get(self, /, *args: _GetP.args, **kwargs: _GetP.kwargs) -> _GetT:
             raise NotImplementedError
 
 
 def _add_get(
     get: Callable[_GetP, _GetT],
+    /,
 ) -> Callable[
     [Callable[_P, _T]],
     _CallableWithGet[_P, _T, _GetP, _GetT],
 ]:
-    def decorator(func: Callable[_P, _T]) -> _CallableWithGet[_P, _T, _GetP, _GetT]:
+    def decorator(func: Callable[_P, _T], /) -> _CallableWithGet[_P, _T, _GetP, _GetT]:
         func.get = get  # type: ignore[attr-defined]
         return func  # type: ignore[return-value]
 
     return decorator
 
 
-def _pytree_node_registry_get(  # pylint: disable=too-many-return-statements
+@overload
+def pytree_node_registry_get(
     cls: type,
+    /,
     *,
     namespace: str = '',
-) -> PyTreeNodeRegistryEntry | None:
-    if namespace is not __GLOBAL_NAMESPACE and namespace != '':
+) -> PyTreeNodeRegistryEntry | None: ...
+
+
+@overload
+def pytree_node_registry_get(
+    cls: None = None,
+    /,
+    *,
+    namespace: str = '',
+) -> dict[type, PyTreeNodeRegistryEntry]: ...
+
+
+# pylint: disable-next=too-many-return-statements,too-many-branches
+def pytree_node_registry_get(  # noqa: C901
+    cls: type | None = None,
+    /,
+    *,
+    namespace: str = '',
+) -> dict[type, PyTreeNodeRegistryEntry] | PyTreeNodeRegistryEntry | None:
+    """Lookup the pytree node registry.
+
+    >>> register_pytree_node.get()  # doctest: +IGNORE_WHITESPACE,ELLIPSIS
+    {
+        <class 'NoneType'>: PyTreeNodeRegistryEntry(
+            type=<class 'NoneType'>,
+            flatten_func=<function ...>,
+            unflatten_func=<function ...>,
+            path_entry_type=<class 'optree.PyTreeEntry'>,
+            kind=<PyTreeKind.NONE: 2>,
+            namespace=''
+        ),
+        <class 'tuple'>: PyTreeNodeRegistryEntry(
+            type=<class 'tuple'>,
+            flatten_func=<function ...>,
+            unflatten_func=<function ...>,
+            path_entry_type=<class 'optree.SequenceEntry'>,
+            kind=<PyTreeKind.TUPLE: 3>,
+            namespace=''
+        ),
+        <class 'list'>: PyTreeNodeRegistryEntry(
+            type=<class 'list'>,
+            flatten_func=<function ...>,
+            unflatten_func=<function ...>,
+            path_entry_type=<class 'optree.SequenceEntry'>,
+            kind=<PyTreeKind.LIST: 4>,
+            namespace=''
+        ),
+        ...
+    }
+    >>> register_pytree_node.get(defaultdict)  # doctest: +IGNORE_WHITESPACE,ELLIPSIS
+    PyTreeNodeRegistryEntry(
+        type=<class 'collections.defaultdict'>,
+        flatten_func=<function ...>,
+        unflatten_func=<function ...>,
+        path_entry_type=<class 'optree.MappingEntry'>,
+        kind=<PyTreeKind.DEFAULTDICT: 8>,
+        namespace=''
+    )
+    >>> register_pytree_node.get(frozenset)  # frozenset is considered as a leaf node
+    None
+
+    Args:
+        cls (type or None, optional): The class of the pytree node to retrieve. If not provided, all
+            the registered pytree nodes in the namespace are returned.
+        namespace (str, optional): The namespace of the registry to retrieve. If not provided, the
+            global namespace is used.
+
+    Returns:
+        If the ``cls`` is not provided, a dictionary of all the registered pytree nodes in the
+        namespace is returned. If the ``cls`` is provided, the corresponding registry entry is
+        returned if the ``cls`` is registered as a pytree node. Otherwise, :data:`None` is returned,
+        i.e., the ``cls`` is represented as a leaf node.
+    """
+    if namespace is __GLOBAL_NAMESPACE:
+        namespace = ''
+    if (
+        cls is not None
+        and cls is not namedtuple  # noqa: PYI024
+        and not inspect.isclass(cls)
+    ):
+        raise TypeError(f'Expected a class or None, got {cls!r}.')  # pragma: !=3.9 cover
+    if not isinstance(namespace, str):
+        raise TypeError(  # pragma: !=3.9 cover
+            f'The namespace must be a string, got {namespace!r}.',
+        )
+
+    if cls is None:
+        namespaces = frozenset({namespace, ''})
+        with __REGISTRY_LOCK:
+            registry = {
+                handler.type: handler
+                for handler in _NODETYPE_REGISTRY.values()
+                if handler.namespace in namespaces
+            }
+        if _C.is_dict_insertion_ordered(namespace):
+            registry[dict] = _DICT_INSERTION_ORDERED_REGISTRY_ENTRY
+            registry[defaultdict] = _DEFAULTDICT_INSERTION_ORDERED_REGISTRY_ENTRY
+        return registry
+
+    if namespace != '':
         handler = _NODETYPE_REGISTRY.get((namespace, cls))
         if handler is not None:
             return handler
@@ -162,15 +244,16 @@ def _pytree_node_registry_get(  # pylint: disable=too-many-return-statements
     if handler is not None:
         return handler
     if is_structseq_class(cls):
-        return _NODETYPE_REGISTRY.get(structseq)
+        return _NODETYPE_REGISTRY.get(StructSequence)
     if is_namedtuple_class(cls):
         return _NODETYPE_REGISTRY.get(namedtuple)  # type: ignore[call-overload] # noqa: PYI024
     return None
 
 
-@_add_get(_pytree_node_registry_get)
+@_add_get(pytree_node_registry_get)
 def register_pytree_node(
     cls: type[Collection[T]],
+    /,
     flatten_func: FlattenFunc[T],
     unflatten_func: UnflattenFunc[T],
     *,
@@ -328,15 +411,13 @@ def register_pytree_node(
     return cls
 
 
-del _pytree_node_registry_get
-
-
-CustomTreeNodeType: TypeAlias = Type[CustomTreeNode[T]]
+del pytree_node_registry_get, _add_get
 
 
 @overload
 def register_pytree_node_class(
     cls: str | None = None,
+    /,
     *,
     path_entry_type: type[PyTreeEntry] | None = None,
     namespace: str | None = None,
@@ -346,6 +427,7 @@ def register_pytree_node_class(
 @overload
 def register_pytree_node_class(
     cls: CustomTreeNodeType,
+    /,
     *,
     path_entry_type: type[PyTreeEntry] | None,
     namespace: str,
@@ -354,6 +436,7 @@ def register_pytree_node_class(
 
 def register_pytree_node_class(  # noqa: C901
     cls: CustomTreeNodeType | str | None = None,
+    /,
     *,
     path_entry_type: type[PyTreeEntry] | None = None,
     namespace: str | None = None,
@@ -391,7 +474,9 @@ def register_pytree_node_class(  # noqa: C901
         ValueError: If the type is already registered in the registry.
 
     This function is a thin wrapper around :func:`register_pytree_node`, and provides a
-    class-oriented interface::
+    class-oriented interface:
+
+    .. code-block:: python
 
         @register_pytree_node_class(namespace='foo')
         class Special:
@@ -424,11 +509,7 @@ def register_pytree_node_class(  # noqa: C901
             raise ValueError('Cannot specify `namespace` when the first argument is a string.')
         if cls == '':
             raise ValueError('The namespace cannot be an empty string.')
-        return functools.partial(
-            register_pytree_node_class,
-            path_entry_type=path_entry_type,
-            namespace=cls,
-        )  # type: ignore[return-value]
+        cls, namespace = None, cls
 
     if namespace is None:
         raise ValueError('Must specify `namespace` when the first argument is a class.')
@@ -438,20 +519,26 @@ def register_pytree_node_class(  # noqa: C901
         raise ValueError('The namespace cannot be an empty string.')
 
     if cls is None:
-        return functools.partial(
-            register_pytree_node_class,
-            path_entry_type=path_entry_type,
-            namespace=namespace,
-        )  # type: ignore[return-value]
+
+        def decorator(cls: CustomTreeNodeType, /) -> CustomTreeNodeType:
+            return register_pytree_node_class(
+                cls,
+                path_entry_type=path_entry_type,
+                namespace=namespace,
+            )
+
+        return decorator
+
     if not inspect.isclass(cls):
         raise TypeError(f'Expected a class, got {cls!r}.')
     if path_entry_type is None:
         path_entry_type = getattr(cls, 'TREE_PATH_ENTRY_TYPE', AutoEntry)
     if not (inspect.isclass(path_entry_type) and issubclass(path_entry_type, PyTreeEntry)):
         raise TypeError(f'Expected a subclass of PyTreeEntry, got {path_entry_type!r}.')
+
     register_pytree_node(
         cls,
-        methodcaller('tree_flatten'),  # type: ignore[arg-type]
+        methodcaller('tree_flatten'),
         cls.tree_unflatten,
         path_entry_type=path_entry_type,
         namespace=namespace,
@@ -459,7 +546,7 @@ def register_pytree_node_class(  # noqa: C901
     return cls
 
 
-def unregister_pytree_node(cls: type, *, namespace: str) -> PyTreeNodeRegistryEntry:
+def unregister_pytree_node(cls: type, /, *, namespace: str) -> PyTreeNodeRegistryEntry:
     """Remove a type from the pytree node registry.
 
     See also :func:`register_pytree_node` and :func:`register_pytree_node_class`.
@@ -513,7 +600,7 @@ def unregister_pytree_node(cls: type, *, namespace: str) -> PyTreeNodeRegistryEn
 
 
 @contextlib.contextmanager
-def dict_insertion_ordered(mode: bool, *, namespace: str) -> Generator[None]:
+def dict_insertion_ordered(mode: bool, /, *, namespace: str) -> Generator[None]:
     """Context manager to temporarily set the dictionary sorting mode.
 
     This context manager is used to temporarily set the dictionary sorting mode for a specific
@@ -559,70 +646,85 @@ def dict_insertion_ordered(mode: bool, *, namespace: str) -> Generator[None]:
             _C.set_dict_insertion_ordered(prev, namespace)
 
 
-def _sorted_items(items: Iterable[tuple[KT, VT]]) -> list[tuple[KT, VT]]:
+def _sorted_items(items: Iterable[tuple[KT, VT]], /) -> list[tuple[KT, VT]]:
     return total_order_sorted(items, key=itemgetter(0))
 
 
-def _none_flatten(_: None) -> tuple[tuple[()], None]:
+def _none_flatten(_: None, /) -> tuple[tuple[()], None]:
     return (), None
 
 
-def _none_unflatten(_: None, children: Iterable[Any]) -> None:
+def _none_unflatten(_: None, children: Iterable[Any], /) -> None:
     sentinel = object()
     if next(iter(children), sentinel) is not sentinel:
         raise ValueError('Expected no children.')
 
 
-def _tuple_flatten(tup: tuple[T, ...]) -> tuple[tuple[T, ...], None]:
+def _tuple_flatten(tup: tuple[T, ...], /) -> tuple[tuple[T, ...], None]:
     return tup, None
 
 
-def _tuple_unflatten(_: None, children: Iterable[T]) -> tuple[T, ...]:
+def _tuple_unflatten(_: None, children: Iterable[T], /) -> tuple[T, ...]:
     return tuple(children)
 
 
-def _list_flatten(lst: list[T]) -> tuple[list[T], None]:
+def _list_flatten(lst: list[T], /) -> tuple[list[T], None]:
     return lst, None
 
 
-def _list_unflatten(_: None, children: Iterable[T]) -> list[T]:
+def _list_unflatten(_: None, children: Iterable[T], /) -> list[T]:
     return list(children)
 
 
-def _dict_flatten(dct: dict[KT, VT]) -> tuple[tuple[VT, ...], list[KT], tuple[KT, ...]]:
+def _dict_flatten(dct: dict[KT, VT], /) -> tuple[tuple[VT, ...], list[KT], tuple[KT, ...]]:
     keys, values = unzip2(_sorted_items(dct.items()))
     return values, list(keys), keys
 
 
-def _dict_unflatten(keys: list[KT], values: Iterable[VT]) -> dict[KT, VT]:
+def _dict_unflatten(keys: list[KT], values: Iterable[VT], /) -> dict[KT, VT]:
     return dict(safe_zip(keys, values))
 
 
 def _dict_insertion_ordered_flatten(
     dct: dict[KT, VT],
-) -> tuple[tuple[VT, ...], list[KT], tuple[KT, ...]]:
+    /,
+) -> tuple[
+    tuple[VT, ...],
+    list[KT],
+    tuple[KT, ...],
+]:
     keys, values = unzip2(dct.items())
     return values, list(keys), keys
 
 
-def _dict_insertion_ordered_unflatten(keys: list[KT], values: Iterable[VT]) -> dict[KT, VT]:
+def _dict_insertion_ordered_unflatten(keys: list[KT], values: Iterable[VT], /) -> dict[KT, VT]:
     return dict(safe_zip(keys, values))
 
 
 def _ordereddict_flatten(
     dct: OrderedDict[KT, VT],
-) -> tuple[tuple[VT, ...], list[KT], tuple[KT, ...]]:
+    /,
+) -> tuple[
+    tuple[VT, ...],
+    list[KT],
+    tuple[KT, ...],
+]:
     keys, values = unzip2(dct.items())
     return values, list(keys), keys
 
 
-def _ordereddict_unflatten(keys: list[KT], values: Iterable[VT]) -> OrderedDict[KT, VT]:
+def _ordereddict_unflatten(keys: list[KT], values: Iterable[VT], /) -> OrderedDict[KT, VT]:
     return OrderedDict(safe_zip(keys, values))
 
 
 def _defaultdict_flatten(
     dct: defaultdict[KT, VT],
-) -> tuple[tuple[VT, ...], tuple[Callable[[], VT] | None, list[KT]], tuple[KT, ...]]:
+    /,
+) -> tuple[
+    tuple[VT, ...],
+    tuple[Callable[[], VT] | None, list[KT]],
+    tuple[KT, ...],
+]:
     values, keys, entries = _dict_flatten(dct)
     return values, (dct.default_factory, keys), entries
 
@@ -630,6 +732,7 @@ def _defaultdict_flatten(
 def _defaultdict_unflatten(
     metadata: tuple[Callable[[], VT], list[KT]],
     values: Iterable[VT],
+    /,
 ) -> defaultdict[KT, VT]:
     default_factory, keys = metadata
     return defaultdict(default_factory, _dict_unflatten(keys, values))
@@ -637,7 +740,12 @@ def _defaultdict_unflatten(
 
 def _defaultdict_insertion_ordered_flatten(
     dct: defaultdict[KT, VT],
-) -> tuple[tuple[VT, ...], tuple[Callable[[], VT] | None, list[KT]], tuple[KT, ...]]:
+    /,
+) -> tuple[
+    tuple[VT, ...],
+    tuple[Callable[[], VT] | None, list[KT]],
+    tuple[KT, ...],
+]:
     values, keys, entries = _dict_insertion_ordered_flatten(dct)
     return values, (dct.default_factory, keys), entries
 
@@ -645,32 +753,38 @@ def _defaultdict_insertion_ordered_flatten(
 def _defaultdict_insertion_ordered_unflatten(
     metadata: tuple[Callable[[], VT], list[KT]],
     values: Iterable[VT],
+    /,
 ) -> defaultdict[KT, VT]:
     default_factory, keys = metadata
     return defaultdict(default_factory, _dict_insertion_ordered_unflatten(keys, values))
 
 
-def _deque_flatten(deq: deque[T]) -> tuple[deque[T], int | None]:
+def _deque_flatten(deq: deque[T], /) -> tuple[deque[T], int | None]:
     return deq, deq.maxlen
 
 
-def _deque_unflatten(maxlen: int | None, children: Iterable[T]) -> deque[T]:
+def _deque_unflatten(maxlen: int | None, children: Iterable[T], /) -> deque[T]:
     return deque(children, maxlen=maxlen)
 
 
-def _namedtuple_flatten(tup: NamedTuple[T]) -> tuple[tuple[T, ...], type[NamedTuple[T]]]:  # type: ignore[type-arg]
+def _namedtuple_flatten(tup: NamedTuple[T], /) -> tuple[tuple[T, ...], type[NamedTuple[T]]]:  # type: ignore[type-arg]
     return tup, type(tup)
 
 
-def _namedtuple_unflatten(cls: type[NamedTuple[T]], children: Iterable[T]) -> NamedTuple[T]:  # type: ignore[type-arg]
+# pylint: disable-next=line-too-long
+def _namedtuple_unflatten(cls: type[NamedTuple[T]], children: Iterable[T], /) -> NamedTuple[T]:  # type: ignore[type-arg]
     return cls(*children)  # type: ignore[call-overload]
 
 
-def _structseq_flatten(seq: structseq[T]) -> tuple[tuple[T, ...], type[structseq[T]]]:
+def _structseq_flatten(seq: StructSequence[T], /) -> tuple[tuple[T, ...], type[StructSequence[T]]]:
     return seq, type(seq)
 
 
-def _structseq_unflatten(cls: type[structseq[T]], children: Iterable[T]) -> structseq[T]:
+def _structseq_unflatten(
+    cls: type[StructSequence[T]],
+    children: Iterable[T],
+    /,
+) -> StructSequence[T]:
     return cls(children)
 
 
@@ -731,8 +845,8 @@ _NODETYPE_REGISTRY: dict[type | tuple[str, type], PyTreeNodeRegistryEntry] = {
         path_entry_type=SequenceEntry,
         kind=PyTreeKind.DEQUE,
     ),
-    structseq: PyTreeNodeRegistryEntry(
-        structseq,
+    StructSequence: PyTreeNodeRegistryEntry(
+        StructSequence,
         _structseq_flatten,
         _structseq_unflatten,
         path_entry_type=StructSequenceEntry,
@@ -755,147 +869,3 @@ _DEFAULTDICT_INSERTION_ORDERED_REGISTRY_ENTRY = PyTreeNodeRegistryEntry(
     path_entry_type=MappingEntry,
     kind=PyTreeKind.DEFAULTDICT,
 )
-
-
-####################################################################################################
-
-with warnings.catch_warnings():
-    warnings.filterwarnings('ignore', category=FutureWarning, module=__name__, append=False)
-
-    @deprecated(
-        'The function `_sorted_keys` is deprecated and will be removed in a future version.',
-        category=FutureWarning,
-    )
-    def _sorted_keys(dct: dict[KT, VT]) -> list[KT]:
-        return total_order_sorted(dct)
-
-    @deprecated(
-        'The key path API is deprecated and will be removed in a future version. '
-        'Please use the accessor API instead.',
-        category=FutureWarning,
-    )
-    class KeyPathEntry(NamedTuple):  # pylint: disable=missing-class-docstring
-        key: Any
-
-        def __add__(self, other: object) -> KeyPath:
-            if isinstance(other, KeyPathEntry):
-                return KeyPath((self, other))
-            if isinstance(other, KeyPath):
-                return KeyPath((self, *other.keys))
-            return NotImplemented
-
-        def __eq__(self, other: object) -> bool:
-            return isinstance(other, self.__class__) and self.key == other.key
-
-        def pprint(self) -> str:
-            """Pretty name of the key path entry."""
-            raise NotImplementedError
-
-    @deprecated(
-        'The key path API is deprecated and will be removed in a future version. '
-        'Please use the accessor API instead.',
-        category=FutureWarning,
-    )
-    class KeyPath(NamedTuple):  # pylint: disable=missing-class-docstring
-        keys: tuple[KeyPathEntry, ...] = ()
-
-        def __add__(self, other: object) -> KeyPath:
-            if isinstance(other, KeyPathEntry):
-                return KeyPath((*self.keys, other))
-            if isinstance(other, KeyPath):
-                return KeyPath(self.keys + other.keys)
-            return NotImplemented
-
-        def __eq__(self, other: object) -> bool:
-            return isinstance(other, KeyPath) and self.keys == other.keys
-
-        def pprint(self) -> str:
-            """Pretty name of the key path."""
-            if not self.keys:
-                return ' tree root'
-            return ''.join(k.pprint() for k in self.keys)
-
-    @deprecated(
-        'The key path API is deprecated and will be removed in a future version. '
-        'Please use the accessor API instead.',
-        category=FutureWarning,
-    )
-    class GetitemKeyPathEntry(KeyPathEntry):
-        """The key path entry class for sequences and dictionaries."""
-
-        def pprint(self) -> str:
-            """Pretty name of the key path entry."""
-            return f'[{self.key!r}]'
-
-    @deprecated(
-        'The key path API is deprecated and will be removed in a future version. '
-        'Please use the accessor API instead.',
-        category=FutureWarning,
-    )
-    class AttributeKeyPathEntry(KeyPathEntry):
-        """The key path entry class for namedtuples."""
-
-        def pprint(self) -> str:
-            """Pretty name of the key path entry."""
-            return f'.{self.key}'
-
-    @deprecated(
-        'The key path API is deprecated and will be removed in a future version. '
-        'Please use the accessor API instead.',
-        category=FutureWarning,
-    )
-    class FlattenedKeyPathEntry(KeyPathEntry):  # fallback
-        """The fallback key path entry class."""
-
-        def pprint(self) -> str:
-            """Pretty name of the key path entry."""
-            return f'[<flat index {self.key}>]'
-
-    KeyPathHandler = Callable[[Collection[T]], Sequence[KeyPathEntry]]
-    _KEYPATH_REGISTRY: dict[type[Collection], KeyPathHandler] = {}
-
-    @deprecated(
-        'The key path API is deprecated and will be removed in a future version. '
-        'Please use the accessor API instead.',
-        category=FutureWarning,
-    )
-    @_add_get(_KEYPATH_REGISTRY.get)
-    def register_keypaths(
-        cls: type[Collection[T]],
-        handler: KeyPathHandler[T],
-    ) -> KeyPathHandler[T]:
-        """Register a key path handler for a custom pytree node type."""
-        if not inspect.isclass(cls):
-            raise TypeError(f'Expected a class, got {cls!r}.')
-        if cls in _KEYPATH_REGISTRY:
-            raise ValueError(f'Key path handler for {cls!r} has already been registered.')
-
-        _KEYPATH_REGISTRY[cls] = handler
-        return handler
-
-    register_keypaths(
-        tuple,
-        lambda tup: list(map(GetitemKeyPathEntry, range(len(tup)))),
-    )
-    register_keypaths(
-        list,
-        lambda lst: list(map(GetitemKeyPathEntry, range(len(lst)))),
-    )
-    register_keypaths(
-        dict,
-        lambda dct: list(map(GetitemKeyPathEntry, _sorted_keys(dct))),  # type: ignore[arg-type]
-    )
-    register_keypaths(
-        OrderedDict,
-        lambda odct: list(map(GetitemKeyPathEntry, odct)),
-    )
-    register_keypaths(
-        defaultdict,
-        lambda ddct: list(map(GetitemKeyPathEntry, _sorted_keys(ddct))),  # type: ignore[arg-type]
-    )
-    register_keypaths(
-        deque,
-        lambda dq: list(map(GetitemKeyPathEntry, range(len(dq)))),
-    )
-
-del _add_get
